﻿using System;
using System.Collections.Generic;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Threading.Tasks;

namespace TestClient
{
    /// <summary>
    /// 프로그램
    /// </summary>
    class Program
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 프로그램 시작하기 - Main()

        /// <summary>
        /// 프로그램 시작하기
        /// </summary>
        static void Main()
        {
            // 포트 5569는 서버 설정과 일치시킨다.
            using(HttpClient client = GetHTTPClient("http://localhost:5569/"))
            {
                List<TestModel> list;
                TestModel       item;
                string          status;

                Console.WriteLine();
                Console.WriteLine("리스트를 조회합니다.");
                Console.WriteLine("--------------------------------------------------");

                list = GetList(client).GetAwaiter().GetResult();

                for(int i = 0; i < list.Count; i++)
                {
                    item = list[i];

                    Console.WriteLine($"{item.ID}, {item.Name}");
                }


                Console.WriteLine();
                Console.WriteLine("항목을 조회합니다.");
                Console.WriteLine("--------------------------------------------------");

                item = GetItem(client, 3).GetAwaiter().GetResult();

                Console.WriteLine($"{item?.ID}, {item?.Name}");


                Console.WriteLine();
                Console.WriteLine("값을 추가합니다.");
                Console.WriteLine("--------------------------------------------------");

                status = AddItem(client, new TestModel { ID = 4, Name = "백만원" }).GetAwaiter().GetResult();

                Console.WriteLine(status);


                Console.WriteLine();
                Console.WriteLine("리스트를 조회합니다.");
                Console.WriteLine("--------------------------------------------------");

                list = GetList(client).GetAwaiter().GetResult();

                for(int i = 0; i < list.Count; i++)
                {
                    item = list[i];

                    Console.WriteLine($"{item.ID}, {item.Name}");
                }


                Console.WriteLine("--------------------------------------------------");

                Console.WriteLine("아무 키나 눌러 주시기 바랍니다.");

                Console.ReadKey(true);
            }
        }

        #endregion

        #region HTTP 클라이언트 구하기 - GetHTTPClient(baseAddress)

        /// <summary>
        /// HTTP 클라이언트 구하기
        /// </summary>
        /// <param name="baseAddress">기준 주소</param>
        /// <returns>HTTP 클라이언트</returns>
        private static HttpClient GetHTTPClient(string baseAddress)
        {
            HttpClient client = new HttpClient();

            client.BaseAddress = new Uri(baseAddress);

            client.DefaultRequestHeaders.Accept.Clear();

            client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

            return client;
        }

        #endregion

        #region 리스트 구하기 - GetList(client)

        /// <summary>
        /// 리스트 구하기
        /// </summary>
        /// <param name="client">클라이언트</param>
        /// <returns>리스트</returns>
        private static async Task<List<TestModel>> GetList(HttpClient client)
        {
            string url = string.Format("api/test/");

            HttpResponseMessage response = await client.GetAsync(url);

            if(response.IsSuccessStatusCode)
            {
                List<TestModel> list = await response.Content.ReadAsAsync<List<TestModel>>();

                return list;
            }

            return null;
        }

        #endregion
        #region 항목 구하기 - GetItem(client, id)

        /// <summary>
        /// 항목 구하기
        /// </summary>
        /// <param name="client">클라이언트</param>
        /// <param name="id">ID</param>
        /// <returns>항목</returns>
        private static async Task<TestModel> GetItem(HttpClient client, int id)
        {
            string url = string.Format("api/test/{0}", id);

            HttpResponseMessage response = await client.GetAsync(url);

            if(response.IsSuccessStatusCode)
            {
                TestModel item = await response.Content.ReadAsAsync<TestModel>();

                return item;
            }

            return null;
        }

        #endregion
        #region 항목 추가하기 - AddItem(client, item)

        /// <summary>
        /// 항목 추가하기
        /// </summary>
        /// <param name="client">클라이언트</param>
        /// <param name="item">항목</param>
        /// <returns>처리 결과</returns>
        private static async Task<string> AddItem(HttpClient client, TestModel item)
        {
            string url = string.Format("api/test/");

            var response = await client.PostAsJsonAsync<TestModel>(url, item);

            if(response.IsSuccessStatusCode)
            {
                string result = await response.Content.ReadAsAsync<string>();

                return result;
            }

            return null;
        }

        #endregion
    }
}