﻿using Microsoft.AspNetCore.Mvc;
using System.Linq;
using System.Net;

using TestServer.Models;

namespace TestServer.Controllers
{
    /// <summary>
    /// 테스트 컨트롤러
    /// </summary>
    [ApiController]
    [Route("api/[controller]")]
    public class TestController : Controller
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 리스트 구하기 - GetList()

        /// <summary>
        /// 리스트 구하기
        /// </summary>
        /// <returns>리스트</returns>
        [HttpGet]
        public JsonResult GetList()
        {
            Response.StatusCode = (int)HttpStatusCode.OK;

            return Json(TestRepository.TestDictionary.Values.ToList());
        }

        #endregion
        #region 항목 구하기 - GetItem(id)

        /// <summary>
        /// 항목 구하기
        /// </summary>
        /// <returns>항목</returns>
        [HttpGet("{id:int}")]
        public JsonResult GetItem(int id)
        {
            if(TestRepository.TestDictionary.ContainsKey(id))
            {
                Response.StatusCode = (int)HttpStatusCode.OK;

                return Json(TestRepository.TestDictionary[id]);
            }
            else
            {
                Response.StatusCode = (int)HttpStatusCode.BadRequest;

                return Json(null);
            }
        }

        #endregion
        #region 항목 추가하기 - AddItem(test)

        /// <summary>
        /// 항목 추가하기
        /// </summary>
        /// <param name="test">테스트</param>
        /// <returns>처리 결과</returns>
        [HttpPost]
        public JsonResult Post([FromBody]TestModel test)
        {
            if(ModelState.IsValid)
            {
                if(TestRepository.TestDictionary.ContainsKey(test.ID))
                {
                    Response.StatusCode = (int)HttpStatusCode.BadRequest;

                    return Json("FAILURE");
                }

                TestRepository.TestDictionary.Add(test.ID, test);

                Response.StatusCode = (int)HttpStatusCode.Created;

                return Json("SUCCESS");
            }

            Response.StatusCode = (int)HttpStatusCode.BadRequest;

            return Json("FAILURE");
        }

        #endregion
    }
}